@@ -20,7 +20,9 @@ qacc MatrixT<DIMN, T> operator+(const MatrixT<DIMN, T>& x,
                                 const MatrixT<DIMN, T>& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() + y.em();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x.p[i] + y.p[i];
+  }
   return ret;
 }
 
@@ -29,7 +31,9 @@ qacc MatrixT<DIMN, T> operator-(const MatrixT<DIMN, T>& x,
                                 const MatrixT<DIMN, T>& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() - y.em();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x.p[i] - y.p[i];
+  }
   return ret;
 }
 
@@ -37,7 +41,9 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> operator-(const MatrixT<DIMN, T>& x)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = -x.em();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = -x.p[i];
+  }
   return ret;
 }
 
@@ -46,7 +52,18 @@ qacc MatrixT<DIMN, T> operator*(const MatrixT<DIMN, T>& x,
                                 const MatrixT<DIMN, T>& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() * y.em();
+  MatrixT<DIMN, T> yt = matrix_transpose(y);
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    for (int j = 0; j < DIMN; ++j) {
+      const int jd = j * DIMN;
+      T s = 0.0;
+      for (int k = 0; k < DIMN; ++k) {
+        s += x.p[id + k] * yt.p[jd + k];
+      }
+      ret.p[id + j] = s;
+    }
+  }
   return ret;
 }
 
@@ -54,7 +71,9 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> operator*(const double x, const MatrixT<DIMN, T>& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x * y.em();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x * y.p[i];
+  }
   return ret;
 }
 
@@ -62,23 +81,29 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> operator*(const MatrixT<DIMN, T>& x, const double y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() * y;
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x.p[i] * y;
+  }
   return ret;
 }
 
 template <int DIMN, class T>
-qacc MatrixT<DIMN, T> operator*(const Complex& x, const MatrixT<DIMN, T>& y)
+qacc MatrixT<DIMN, T> operator*(const ComplexD& x, const MatrixT<DIMN, T>& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x * y.em();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x * y.p[i];
+  }
   return ret;
 }
 
 template <int DIMN, class T>
-qacc MatrixT<DIMN, T> operator*(const MatrixT<DIMN, T>& x, const Complex& y)
+qacc MatrixT<DIMN, T> operator*(const MatrixT<DIMN, T>& x, const ComplexD& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() * y;
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x.p[i] * y;
+  }
   return ret;
 }
 
@@ -86,7 +111,9 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> operator/(const MatrixT<DIMN, T>& x, const T& y)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em() / y;
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = x.p[i] / y;
+  }
   return ret;
 }
 
@@ -95,7 +122,14 @@ qacc MvectorT<DIMN, T> operator*(const MatrixT<DIMN, T>& x,
                                  const MvectorT<DIMN, T>& y)
 {
   MvectorT<DIMN, T> ret;
-  ret.em() = x.em() * y.em();
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    T s = 0.0;
+    for (int j = 0; j < DIMN; ++j) {
+      s += x.p[id + j] * y.p[j];
+    }
+    ret.p[i] = s;
+  }
   return ret;
 }
 
@@ -106,7 +140,7 @@ qacc void set_zero(MatrixT<DIMN, T>& m)
 }
 
 template <int DIMN>
-qacc void set_unit(MatrixT<DIMN, float>& m, const Complex& coef = 1.0)
+qacc void set_unit(MatrixT<DIMN, RealF>& m, const ComplexD& coef = 1.0)
 {
   set_zero(m);
   for (int i = 0; i < DIMN; ++i) {
@@ -115,7 +149,7 @@ qacc void set_unit(MatrixT<DIMN, float>& m, const Complex& coef = 1.0)
 }
 
 template <int DIMN>
-qacc void set_unit(MatrixT<DIMN, double>& m, const Complex& coef = 1.0)
+qacc void set_unit(MatrixT<DIMN, RealD>& m, const ComplexD& coef = 1.0)
 {
   set_zero(m);
   for (int i = 0; i < DIMN; ++i) {
@@ -124,7 +158,7 @@ qacc void set_unit(MatrixT<DIMN, double>& m, const Complex& coef = 1.0)
 }
 
 template <int DIMN, class T>
-qacc void set_unit(MatrixT<DIMN, T>& m, const Complex& coef = 1.0)
+qacc void set_unit(MatrixT<DIMN, T>& m, const ComplexD& coef = 1.0)
 {
   set_zero(m);
   for (int i = 0; i < DIMN; ++i) {
@@ -133,28 +167,50 @@ qacc void set_unit(MatrixT<DIMN, T>& m, const Complex& coef = 1.0)
 }
 
 template <int DIMN, class T>
-qacc double qnorm(const MatrixT<DIMN, T>& m)
+qacc RealD qnorm(const MatrixT<DIMN, T>& m)
 {
-  return m.em().squaredNorm();
+  RealD s = 0.0;
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    s += qnorm(m.p[i]);
+  }
+  return s;
 }
 
 template <int DIMN, class T>
-qacc Complex matrix_trace(const MatrixT<DIMN, T>& x)
+qacc ComplexD matrix_trace(const MatrixT<DIMN, T>& x)
 {
-  return x.em().trace();
+  ComplexD s = 0.0;
+  for (int i = 0; i < DIMN; ++i) {
+    s += x.p[i * (DIMN + 1)];
+  }
+  return s;
 }
 
 template <int DIMN, class T>
-qacc Complex matrix_trace(const MatrixT<DIMN, T>& x, const MatrixT<DIMN, T>& y)
+qacc ComplexD matrix_trace(const MatrixT<DIMN, T>& x, const MatrixT<DIMN, T>& y)
 {
-  return (x.em() * y.em()).trace();
+  MatrixT<DIMN, T> yt = matrix_transpose(y);
+  ComplexD s = 0.0;
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    for (int j = 0; j < DIMN; ++j) {
+      s += x.p[id + j] * yt.p[id + j];
+    }
+  }
+  return s;
 }
 
 template <int DIMN, class T>
 qacc MatrixT<DIMN, T> matrix_adjoint(const MatrixT<DIMN, T>& x)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em().adjoint();
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    for (int j = 0; j < DIMN; ++j) {
+      const int jd = j * DIMN;
+      ret.p[jd + i] = qconj(x.p[id + j]);
+    }
+  }
   return ret;
 }
 
@@ -162,7 +218,13 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> matrix_transpose(const MatrixT<DIMN, T>& x)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em().transpose();
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    for (int j = 0; j < DIMN; ++j) {
+      const int jd = j * DIMN;
+      ret.p[jd + i] = x.p[id + j];
+    }
+  }
   return ret;
 }
 
@@ -170,7 +232,9 @@ template <int DIMN, class T>
 qacc MatrixT<DIMN, T> matrix_conjugate(const MatrixT<DIMN, T>& x)
 {
   MatrixT<DIMN, T> ret;
-  ret.em() = x.em().conjugate();
+  for (int i = 0; i < DIMN * DIMN; ++i) {
+    ret.p[i] = qconj(x.p[i]);
+  }
   return ret;
 }
 
@@ -275,14 +339,6 @@ qacc void matrix_lubksb(MvectorT<DIMN, T>& b, const MatrixT<DIMN, T>& a,
   }
 }
 
-template <int DIMN, class T>
-MatrixT<DIMN, T> matrix_inverse_eigen(const MatrixT<DIMN, T>& x)
-{
-  MatrixT<DIMN, T> ret;
-  ret.em() = x.em().inverse();
-  return ret;
-}
-
 template <int DIMN, class T>
 qacc MatrixT<DIMN, T> matrix_inverse(const MatrixT<DIMN, T>& x)
 {
@@ -303,12 +359,6 @@ qacc MatrixT<DIMN, T> matrix_inverse(const MatrixT<DIMN, T>& x)
   return ret;
 }
 
-template <int DIMN, class T>
-T matrix_determinant_eigen(const MatrixT<DIMN, T>& x)
-{
-  return x.em().determinant();
-}
-
 template <int DIMN, class T>
 qacc T matrix_determinant(const MatrixT<DIMN, T>& x)
 {
@@ -338,35 +388,35 @@ struct API SpinMatrixConstantsT {
   //
   qacc void init()
   {
-    Complex ii(0.0, 1.0);
+    ComplexD ii(0.0, 1.0);
     // TIMER_VERBOSE("SpinMatrixConstants::init()");
-    unit.em() << 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1;
+    unit << 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1;
     // gamma_x
-    gammas[0].em() << 0, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, 0;
+    gammas[0] << 0, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, 0;
     // gamma_y
-    gammas[1].em() << 0, 0, 0, (ComplexT<T>)-ii, 0, 0, (ComplexT<T>)ii, 0, 0, (ComplexT<T>)ii, 0, 0, (ComplexT<T>)-ii,
-        0, 0, 0;
+    gammas[1] << 0, 0, 0, (ComplexT<T>)-ii, 0, 0, (ComplexT<T>)ii, 0, 0,
+        (ComplexT<T>)ii, 0, 0, (ComplexT<T>)-ii, 0, 0, 0;
     // gamma_z
-    gammas[2].em() << 0, 0, 1, 0, 0, 0, 0, -1, -1, 0, 0, 0, 0, 1, 0, 0;
+    gammas[2] << 0, 0, 1, 0, 0, 0, 0, -1, -1, 0, 0, 0, 0, 1, 0, 0;
     gammas[0] *= (ComplexT<T>)(-ii);
     gammas[1] *= (ComplexT<T>)(-ii);
     gammas[2] *= (ComplexT<T>)(-ii);
     // gamma_t
-    gammas[3].em() << 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0;
+    gammas[3] << 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0;
     //
     cps_gammas[0] = -gammas[0];
     cps_gammas[1] = gammas[1];
     cps_gammas[2] = -gammas[2];
     cps_gammas[3] = gammas[3];
     // gamma_5 = gamma_x * gamma_y * gamma_z * gamma_t;
-    gamma5.em() << 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, 0, 0, -1;
+    gamma5 << 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, 0, 0, -1;
     // Sigma_x
-    cap_sigmas[0].em() << 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0;
+    cap_sigmas[0] << 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0;
     // Sigma_y
-    cap_sigmas[1].em() << 0, (ComplexT<T>)-ii, 0, 0, (ComplexT<T>)ii, 0, 0, 0, 0, 0, 0, (ComplexT<T>)-ii, 0,
-        0, (ComplexT<T>)ii, 0;
+    cap_sigmas[1] << 0, (ComplexT<T>)-ii, 0, 0, (ComplexT<T>)ii, 0, 0, 0, 0, 0,
+        0, (ComplexT<T>)-ii, 0, 0, (ComplexT<T>)ii, 0;
     // Sigma_z
-    cap_sigmas[2].em() << 1, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1, 0, 0, 0, 0, -1;
+    cap_sigmas[2] << 1, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1, 0, 0, 0, 0, -1;
     //
     cps_cap_sigmas[0] = -cap_sigmas[0];
     cps_cap_sigmas[1] = cap_sigmas[1];
@@ -408,10 +458,10 @@ struct API SpinMatrixConstantsT {
     }
   }
   //
-  API static const box_acc<SpinMatrixConstantsT<T> >& get_instance_box()
+  API static const box_acc<SpinMatrixConstantsT<T>>& get_instance_box()
   {
-    static box_acc<SpinMatrixConstantsT<T> > smcs =
-        box_acc<SpinMatrixConstantsT<T> >(SpinMatrixConstantsT<T>());
+    static box_acc<SpinMatrixConstantsT<T>> smcs =
+        box_acc<SpinMatrixConstantsT<T>>(SpinMatrixConstantsT<T>());
     return smcs;
   }
   //
@@ -498,9 +548,12 @@ qacc WilsonMatrixT<T> operator*(const ColorMatrixT<T>& cm,
     for (int s2 = 0; s2 < 4; ++s2) {
       for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
         for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
+          const int s1_s2_c2 =
+              s1 * (NUM_COLOR * 4 * NUM_COLOR) + s2 * NUM_COLOR + c2;
           for (int c3 = 0; c3 < NUM_COLOR; ++c3) {
-            ret(s1 * NUM_COLOR + c1, s2 * NUM_COLOR + c2) +=
-                cm(c1, c3) * m(s1 * NUM_COLOR + c3, s2 * NUM_COLOR + c2);
+            ret.p[s1_s2_c2 + c1 * (4 * NUM_COLOR)] +=
+                cm.p[c1 * NUM_COLOR + c3] *
+                m.p[s1_s2_c2 + c3 * (4 * NUM_COLOR)];
           }
         }
       }
@@ -519,9 +572,11 @@ qacc WilsonMatrixT<T> operator*(const WilsonMatrixT<T>& m,
     for (int s2 = 0; s2 < 4; ++s2) {
       for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
         for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
+          const int s1_c1_s2 =
+              (s1 * NUM_COLOR + c1) * (4 * NUM_COLOR) + s2 * NUM_COLOR;
           for (int c3 = 0; c3 < NUM_COLOR; ++c3) {
-            ret(s1 * NUM_COLOR + c1, s2 * NUM_COLOR + c2) +=
-                m(s1 * NUM_COLOR + c1, s2 * NUM_COLOR + c3) * cm(c3, c2);
+            ret.p[s1_c1_s2 + c2] +=
+                m.p[s1_c1_s2 + c3] * cm.p[c3 * NUM_COLOR + c2];
           }
         }
       }
@@ -538,14 +593,18 @@ qacc WilsonMatrixT<T> operator*(const SpinMatrixT<T>& sm,
   set_zero(ret);
   for (int s1 = 0; s1 < 4; ++s1) {
     for (int s3 = 0; s3 < 4; ++s3) {
-      if (sm(s1, s3) == 0.0) {
+      const ComplexT<T>& sm_s1_s3 = sm.p[s1 * 4 + s3];
+      if (sm_s1_s3 == 0.0) {
         continue;
       }
       for (int s2 = 0; s2 < 4; ++s2) {
         for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
+          const int s1_c1_s2 =
+              (s1 * NUM_COLOR + c1) * (4 * NUM_COLOR) + s2 * NUM_COLOR;
+          const int s3_c1_s2 =
+              (s3 * NUM_COLOR + c1) * (4 * NUM_COLOR) + s2 * NUM_COLOR;
           for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
-            ret(s1 * NUM_COLOR + c1, s2 * NUM_COLOR + c2) +=
-                sm(s1, s3) * m(s3 * NUM_COLOR + c1, s2 * NUM_COLOR + c2);
+            ret.p[s1_c1_s2 + c2] += sm_s1_s3 * m.p[s3_c1_s2 + c2];
           }
         }
       }
@@ -562,14 +621,18 @@ qacc WilsonMatrixT<T> operator*(const WilsonMatrixT<T>& m,
   set_zero(ret);
   for (int s1 = 0; s1 < 4; ++s1) {
     for (int s3 = 0; s3 < 4; ++s3) {
-      if (sm(s3, s1) == 0.0) {
+      const ComplexT<T>& sm_s3_s1 = sm.p[s3 * 4 + s1];
+      if (sm_s3_s1 == 0.0) {
         continue;
       }
       for (int s2 = 0; s2 < 4; ++s2) {
-        for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
-          for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
-            ret(s2 * NUM_COLOR + c2, s1 * NUM_COLOR + c1) +=
-                m(s2 * NUM_COLOR + c2, s3 * NUM_COLOR + c1) * sm(s3, s1);
+        for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
+          const int s2_c2_s1 =
+              (s2 * NUM_COLOR + c2) * (4 * NUM_COLOR) + s1 * NUM_COLOR;
+          const int s2_c2_s3 =
+              (s2 * NUM_COLOR + c2) * (4 * NUM_COLOR) + s3 * NUM_COLOR;
+          for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
+            ret.p[s2_c2_s1 + c1] += m.p[s2_c2_s3 + c1] * sm_s3_s1;
           }
         }
       }
@@ -579,16 +642,18 @@ qacc WilsonMatrixT<T> operator*(const WilsonMatrixT<T>& m,
 }
 
 template <class T>
-qacc Complex matrix_trace(const SpinMatrixT<T>& sm, const WilsonMatrixT<T>& m)
+qacc ComplexD matrix_trace(const SpinMatrixT<T>& sm, const WilsonMatrixT<T>& m)
 {
-  Complex ret = 0;
+  ComplexD ret = 0;
   for (int s1 = 0; s1 < 4; ++s1) {
     for (int s3 = 0; s3 < 4; ++s3) {
-      if (sm(s1, s3) == 0.0) {
+      const ComplexT<T>& sm_s1_s3 = sm.p[s1 * 4 + s3];
+      if (sm_s1_s3 == 0.0) {
         continue;
       }
+      const int s3_s1 = s3 * (NUM_COLOR * 4 * NUM_COLOR) + s1 * NUM_COLOR;
       for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
-        ret += sm(s1, s3) * m(s3 * NUM_COLOR + c1, s1 * NUM_COLOR + c1);
+        ret += sm_s1_s3 * m.p[s3_s1 + c1 * (4 * NUM_COLOR + 1)];
       }
     }
   }
@@ -596,7 +661,7 @@ qacc Complex matrix_trace(const SpinMatrixT<T>& sm, const WilsonMatrixT<T>& m)
 }
 
 template <class T>
-qacc Complex matrix_trace(const WilsonMatrixT<T>& m, const SpinMatrixT<T>& sm)
+qacc ComplexD matrix_trace(const WilsonMatrixT<T>& m, const SpinMatrixT<T>& sm)
 {
   return matrix_trace(sm, m);
 }
@@ -610,7 +675,8 @@ qacc WilsonVectorT<T> operator*(const ColorMatrixT<T>& cm,
   for (int s1 = 0; s1 < 4; ++s1) {
     for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
       for (int c2 = 0; c2 < NUM_COLOR; ++c2) {
-        ret(s1 * NUM_COLOR + c1) += cm(c1, c2) * m(s1 * NUM_COLOR + c2);
+        ret.p[s1 * NUM_COLOR + c1] +=
+            cm.p[c1 * NUM_COLOR + c2] * m.p[s1 * NUM_COLOR + c2];
       }
     }
   }
@@ -625,11 +691,13 @@ qacc WilsonVectorT<T> operator*(const SpinMatrixT<T>& sm,
   set_zero(ret);
   for (int s1 = 0; s1 < 4; ++s1) {
     for (int s2 = 0; s2 < 4; ++s2) {
-      if (sm(s1, s2) == 0.0) {
+      const ComplexT<T>& sm_s1_s2 = sm.p[s1 * 4 + s2];
+      if (sm_s1_s2 == 0.0) {
         continue;
       }
       for (int c1 = 0; c1 < NUM_COLOR; ++c1) {
-        ret(s1 * NUM_COLOR + c1) += sm(s1, s2) * m(s2 * NUM_COLOR + c1);
+        ret.p[s1 * NUM_COLOR + c1] +=
+            sm.p[s1 * 4 + s2] * m.p[s2 * NUM_COLOR + c1];
       }
     }
   }
@@ -885,7 +953,16 @@ template <int DIMN, class T>
 std::string show(const MatrixT<DIMN, T>& m)
 {
   std::ostringstream out;
-  out << m.em();
+  out << "[ ";
+  for (int i = 0; i < DIMN; ++i) {
+    const int id = i * DIMN;
+    out << "[ ";
+    for (int j = 0; j < DIMN; ++j) {
+      out << m.p[id + j] << ", ";
+    }
+    out << "], ";
+  }
+  out << "]";
   return out.str();
 }
 
